#include "indigo.h"
#include <stdio.h>
#include <stdlib.h>

static void reportError(int error_code) {
  fprintf(stderr,"Occam API Error: %i\n",error_code);
  abort();
}

int main(int argc, const char** argv) {
  int r;
  int i;
  int dev_index = argc>=2?atoi(argv[1]):0;
  OccamDeviceList* device_list;
  OccamDevice* device;
  OccamImage* image;

  if ((r = occamInitialize()) != OCCAM_API_SUCCESS)
    reportError(r);

  if ((r = occamEnumerateDeviceList(2000, &device_list)) != OCCAM_API_SUCCESS)
    reportError(r);
  printf("%i devices found\n", device_list->entry_count);
  for (i=0;i<device_list->entry_count;++i) {
    printf("device[%i]: cid = %s\n",
	   i,device_list->entries[i].cid);
  }
  if (dev_index<0 || dev_index >= device_list->entry_count) {
    fprintf(stderr,"device index %i out of range\n",dev_index);
    return 1;
  }

  if ((r = occamOpenDevice(device_list->entries[dev_index].cid, &device)) != OCCAM_API_SUCCESS)
    reportError(r);

  for (i=0;i<10;++i) {
    for (;;) {
      r = occamDeviceReadImage(device, &image, 0);
      if (r == OCCAM_API_IMAGE_NOT_AVAILABLE) {
	// ... do other work
	continue;
      }
      if (r != OCCAM_API_SUCCESS)
	reportError(r);
      break;
    }

    printf("read image %i: time_ns = %llu, index = %i, format = %i, width = %i, height = %i\n",
    	   i, (long long unsigned int)image->time_ns, image->index, image->format, image->width, image->height);
    occamFreeImage(image);
  }

  occamCloseDevice(device);
  occamFreeDeviceList(device_list);
  occamShutdown();

  return 0;
}
