#ifndef __OCCAM_INDIGO_API_H__
#define __OCCAM_INDIGO_API_H__

#ifdef __cplusplus
#define OCCAM_SDK_LINKAGE extern "C"
#else // __cplusplus
#define OCCAM_SDK_LINKAGE
#endif // __cplusplus
#ifdef _WIN32
#ifdef OCCAM_SDK_BUILD
#define OCCAM_API OCCAM_SDK_LINKAGE __declspec(dllexport)
#else // OCCAM_SDK_BUILD
#define OCCAM_API OCCAM_SDK_LINKAGE
#endif // OCCAM_SDK_BUILD
#else // _WIN32
#define OCCAM_API OCCAM_SDK_LINKAGE
#endif // _WIN32

#include <stdint.h>

OCCAM_API int occamInitialize();
OCCAM_API int occamShutdown();
OCCAM_API void* occamAlloc(int size);
OCCAM_API void occamFree(void* ptr);

typedef struct _OccamDeviceListEntry {
  char* cid;
} OccamDeviceListEntry;

typedef struct _OccamDeviceList {
  OccamDeviceListEntry* entries;
  int entry_count;
} OccamDeviceList;

OCCAM_API int occamEnumerateDeviceList(int timeout_ms, OccamDeviceList** device_list);
OCCAM_API int occamFreeDeviceList(OccamDeviceList* device_list);

typedef void OccamDevice;
OCCAM_API int occamOpenDevice(const char* cid, OccamDevice** device);
OCCAM_API int occamCloseDevice(OccamDevice* device);

typedef enum _OccamParamType {
  OCCAM_PARAM_INT,
  OCCAM_PARAM_REAL,
  OCCAM_PARAM_STRING
} OccamParamType;

typedef enum _OccamDeviceParam {
  OCCAM_EXPOSURE,
  OCCAM_GAIN,
  OCCAM_AUTO_EXPOSURE,
  OCCAM_AUTO_GAIN,
  OCCAM_SENSOR_WIDTH,
  OCCAM_SENSOR_HEIGHT,
  OCCAM_SENSOR_COUNT,
  OCCAM_COLOR,
  OCCAM_STITCHING_ENABLED,
  OCCAM_TRIGGER_MODE,
  OCCAM_MAX_USB_QUEUED_FRAMES,
  OCCAM_MAX_PROCESSING_QUEUED_FRAMES
} OccamDeviceParam;

typedef struct _OccamDeviceParamEntry {
  OccamDeviceParam id;
  char* name;
  OccamParamType type;
  double min_value;
  double max_value;
  int count;
} OccamDeviceParamEntry;

typedef struct _OccamDeviceParamList {
  OccamDeviceParamEntry* params;
  int param_count;
} OccamDeviceParamList;

OCCAM_API int occamEnumerateDeviceParamList(OccamDevice* device, OccamDeviceParamList** param_list);
OCCAM_API int occamFreeDeviceParamList(OccamDeviceParamList* param_list);
OCCAM_API int occamSetDeviceValuei(OccamDevice* device, OccamDeviceParam id, int value);
OCCAM_API int occamSetDeviceValuer(OccamDevice* device, OccamDeviceParam id, double value);
OCCAM_API int occamSetDeviceValues(OccamDevice* device, OccamDeviceParam id, const char* value);
OCCAM_API int occamSetDeviceValueiv(OccamDevice* device, OccamDeviceParam id, const int* values, int value_count);
OCCAM_API int occamSetDeviceValuerv(OccamDevice* device, OccamDeviceParam id, const double* values, int value_count);
OCCAM_API int occamSetDeviceValuesv(OccamDevice* device, OccamDeviceParam id, const char* const* values, int value_count);
OCCAM_API int occamGetDeviceValuei(OccamDevice* device, OccamDeviceParam id, int* value);
OCCAM_API int occamGetDeviceValuer(OccamDevice* device, OccamDeviceParam id, double* value);
OCCAM_API int occamGetDeviceValues(OccamDevice* device, OccamDeviceParam id, const char** value);
OCCAM_API int occamGetDeviceValueiv(OccamDevice* device, OccamDeviceParam id, int* values, int value_count);
OCCAM_API int occamGetDeviceValuerv(OccamDevice* device, OccamDeviceParam id, double* value, int value_count);
OCCAM_API int occamGetDeviceValuesv(OccamDevice* device, OccamDeviceParam id, const char** value, int value_count);

OCCAM_API int occamWriteRegister(OccamDevice* device, uint32_t addr, uint32_t value);
OCCAM_API int occamReadRegister(OccamDevice* device, uint32_t addr, uint32_t* value);

OCCAM_API int occamWriteStorage(OccamDevice* device, uint32_t addr, uint32_t len, const uint8_t* data);
OCCAM_API int occamReadStorage(OccamDevice* device, uint32_t addr, uint32_t len, uint8_t* data);

typedef enum _OccamImageBackend {
  OCCAM_LOCAL,
  OCCAM_OPENGL
} OccamImageBackend;

typedef enum _OccamImageFormat {
  OCCAM_GRAY8,
  OCCAM_RGB24
} OccamImageFormat;

#define OCCAM_IMAGE_MAX_PLANES 3

typedef struct _OccamImage {
  char* cid;
  uint8_t timescale[16];
  uint64_t time_ns;
  uint32_t index;
  int refcnt;

  OccamImageBackend backend;
  OccamImageFormat format;
  int width;
  int height;

  // local backend
  int step[OCCAM_IMAGE_MAX_PLANES];
  uint8_t* data[OCCAM_IMAGE_MAX_PLANES];

  // opengl backend
  int texture;
} OccamImage;

OCCAM_API int occamDeviceReadImage(OccamDevice* device, OccamImage** image, int block);
OCCAM_API int occamFreeImage(OccamImage* image);
OCCAM_API int occamCopyImage(const OccamImage* image, OccamImage** new_image, int deep_copy);

enum OccamError {
  OCCAM_API_SUCCESS,
  OCCAM_API_GENERIC_ERROR,
  OCCAM_API_NOT_INITIALIZED,
  OCCAM_API_ALREADY_INITIALIZED,
  OCCAM_API_ERROR_ENUMERATING_DEVICES,
  OCCAM_API_NOT_SUPPORTED,
  OCCAM_API_INVALID_PARAMETER,
  OCCAM_API_INVALID_TYPE,
  OCCAM_API_INVALID_COUNT,
  OCCAM_API_INVALID_FORMAT,
  OCCAM_API_IMAGE_NOT_AVAILABLE,
  OCCAM_API_WRITE_ERROR,
  OCCAM_API_READ_ERROR
};

#endif // __OCCAM_INDIGO_API_H__

// Local Variables:
// mode: c++
// End:
