#include "indigo.h"
#include <vector>
#include <string>
#include <sstream>
#include <string.h>
#include <libusb-1.0/libusb.h>

static bool libusb_enumerateDevices(std::vector<std::string>& ret_devices) {
  libusb_device** device_list;
  ssize_t num_devices = libusb_get_device_list(0, &device_list);

  for (int j=0;j<num_devices;++j) {
    libusb_device* dev = device_list[j];
    struct libusb_device_descriptor desc;
    int r = libusb_get_device_descriptor(dev,&desc);
    if (r)
      return false;

    if (desc.idVendor != 0x285e)
      continue;

    libusb_device_handle* handle;
    r = libusb_open(dev, &handle);
    if (r)
      continue;

    char model[1024];
    char serial[1024];
    r = libusb_get_string_descriptor_ascii(handle, desc.iProduct, (unsigned char*)model, sizeof(model));
    r = libusb_get_string_descriptor_ascii(handle, desc.iSerialNumber, (unsigned char*)serial, sizeof(serial));

    std::stringstream cid;
    cid<<model<<":"<<serial;
    ret_devices.push_back(cid.str());

    libusb_close(handle);
  }

  libusb_free_device_list(device_list,1);

  return true;
}

int occamEnumerateDeviceList(int timeout_ms, OccamDeviceList** device_list) {
  std::vector<std::string> ret_devices;
  if (!libusb_enumerateDevices(ret_devices))
    return OCCAM_API_ERROR_ENUMERATING_DEVICES;

  *device_list = new OccamDeviceList;
  (*device_list)->entries = new OccamDeviceListEntry[ret_devices.size()];
  (*device_list)->entry_count = ret_devices.size();
  for (int j=0;j<ret_devices.size();++j)
    (*device_list)->entries[j].cid = strdup(ret_devices[j].c_str());

  return OCCAM_API_SUCCESS;
}

int occamFreeDeviceList(OccamDeviceList* device_list) {
  for (int j=0;j<device_list->entry_count;++j)
    free(device_list->entries[j].cid);
  delete [] device_list->entries;
  delete device_list;
  return OCCAM_API_SUCCESS;
}
