#include "indigo.h"
#include "device_iface.h"
#include <map>
#include <string>
#include <functional>
#include <assert.h>
#include <libusb-1.0/libusb.h>

extern void init_omni5u3mt9v022();
static std::map<std::string, std::function<OccamDeviceBase*(const std::string&)> >* device_ctor_map = 0;

int occamInitialize() {
  int r = libusb_init(NULL);

  if (!device_ctor_map) {
    device_ctor_map = new std::map<std::string, std::function<OccamDeviceBase*(const std::string&)> >;

    init_omni5u3mt9v022();
  }

  return r ? OCCAM_API_NOT_INITIALIZED : OCCAM_API_SUCCESS;
}

int occamShutdown() {
  libusb_exit(0);
  delete device_ctor_map;
  device_ctor_map = 0;
  return OCCAM_API_SUCCESS;
}

void* occamAlloc(int size) {
  return malloc(size);
}

void occamFree(void* ptr) {
  return free(ptr);
}

void registerDevice(const std::string& model,
		    std::function<OccamDeviceBase*(const std::string&)> ctor_fn) {
  assert(device_ctor_map);
  device_ctor_map->insert(std::make_pair(model,ctor_fn));
}

int occamOpenDevice(const char* cid, OccamDevice** device) {
  std::string model(cid);
  std::string::size_type p0 = model.find_first_of(":");
  if (p0 == std::string::npos)
    return OCCAM_API_INVALID_PARAMETER;
  model.erase(model.begin()+p0,model.end());
  auto fi = device_ctor_map->find(model);
  if (fi == device_ctor_map->end())
    return OCCAM_API_INVALID_PARAMETER;
  *device = fi->second(cid);
  if (!*device)
    return OCCAM_API_ERROR_ENUMERATING_DEVICES;
  return OCCAM_API_SUCCESS;
}

int occamCloseDevice(OccamDevice* device) {
  delete (OccamDeviceBase*)device;
  return OCCAM_API_SUCCESS;
}

int occamEnumerateDeviceParamList(OccamDevice* device, OccamDeviceParamList** param_list) {
  return ((OccamDeviceBase*)device)->enumerateDeviceParamList(param_list);
}

int occamFreeDeviceParamList(OccamDeviceParamList* param_list) {
  for (int j=0;j<param_list->param_count;++j)
    free(param_list->params[j].name);
  delete [] param_list->params;
  delete param_list;
  return OCCAM_API_SUCCESS;
}

int occamSetDeviceValuei(OccamDevice* device, OccamDeviceParam id, int value) {
  return ((OccamDeviceBase*)device)->setDeviceValuei(id, value);
}

int occamSetDeviceValuer(OccamDevice* device, OccamDeviceParam id, double value) {
  return ((OccamDeviceBase*)device)->setDeviceValuer(id, value);
}

int occamSetDeviceValues(OccamDevice* device, OccamDeviceParam id, const char* value) {
  return ((OccamDeviceBase*)device)->setDeviceValues(id, value);
}

int occamSetDeviceValueiv(OccamDevice* device, OccamDeviceParam id, const int* values, int value_count) {
  return ((OccamDeviceBase*)device)->setDeviceValueiv(id, values, value_count);
}

int occamSetDeviceValuerv(OccamDevice* device, OccamDeviceParam id, const double* values, int value_count) {
  return ((OccamDeviceBase*)device)->setDeviceValuerv(id, values, value_count);
}

int occamSetDeviceValuesv(OccamDevice* device, OccamDeviceParam id, const char* const* values, int value_count) {
  return ((OccamDeviceBase*)device)->setDeviceValuesv(id, values, value_count);
}

int occamGetDeviceValuei(OccamDevice* device, OccamDeviceParam id, int* value) {
  return ((OccamDeviceBase*)device)->getDeviceValuei(id, value);
}

int occamGetDeviceValuer(OccamDevice* device, OccamDeviceParam id, double* value) {
  return ((OccamDeviceBase*)device)->getDeviceValuer(id, value);
}

int occamGetDeviceValues(OccamDevice* device, OccamDeviceParam id, char** value) {
  return ((OccamDeviceBase*)device)->getDeviceValues(id, value);
}

int occamGetDeviceValueiv(OccamDevice* device, OccamDeviceParam id, int* values, int value_count) {
  return ((OccamDeviceBase*)device)->getDeviceValueiv(id, values, value_count);
}

int occamGetDeviceValuerv(OccamDevice* device, OccamDeviceParam id, double* values, int value_count) {
  return ((OccamDeviceBase*)device)->getDeviceValuerv(id, values, value_count);
}

int occamGetDeviceValuesv(OccamDevice* device, OccamDeviceParam id, char** values, int value_count) {
  return ((OccamDeviceBase*)device)->getDeviceValuesv(id, values, value_count);
}

int occamWriteRegister(OccamDevice* device, uint32_t addr, uint32_t value) {
  return ((OccamDeviceBase*)device)->writeRegister(addr, value);
}

int occamReadRegister(OccamDevice* device, uint32_t addr, uint32_t* value) {
  return ((OccamDeviceBase*)device)->readRegister(addr, value);
}

int occamWriteStorage(OccamDevice* device, uint32_t addr, uint32_t len, const uint8_t* data) {
  return ((OccamDeviceBase*)device)->writeStorage(addr, len, data);
}

int occamReadStorage(OccamDevice* device, uint32_t addr, uint32_t len, uint8_t* data) {
  return ((OccamDeviceBase*)device)->readStorage(addr, len, data);
}

int occamDeviceReadImage(OccamDevice* device, OccamImage** image, int block) {
  return ((OccamDeviceBase*)device)->readImage(image, block);
}
