#include "device_iface.h"
#include "omni_libusb.h"
#include "threaded_filter.h"
#include "simple_cpu_blend.h"

class OccamDevice_omni5u3mt9v022 : public OmniDevice {
  bool stitching_enabled;
  ThreadedImageFilter cpu_pipeline;

  int get_exposure() {
    return int(program("w0xcc02=1;w0xcc01=0xb8;r0x0b")[0]);
  }
  int set_exposure(int value) {
    program("w0xcc02=1;w0xcc01=0xb8;w0x0b=?",value);
  }
  int get_auto_exposure() {
    return program("w0xcc02=1;w0xcc01=0xb8;r0xaf")[0]&1?true:false;
  }
  void set_auto_exposure(int value) {
    program("w0xcc02=1;w0xcc01=0xb8;W0xaf(1)=?",value?1:0);
  }
  int get_gain() {
    return int(program("w0xcc02=1;w0xcc01=0xb8;r0x35")[0]);
  }
  void set_gain(int value) {
    value = std::min(std::max(1,value),64);
    program("w0xcc02=1;w0xcc01=0xb8;W0x35(127)=?",value);
  }
  int get_auto_gain() {
    return program("w0xcc02=1;w0xcc01=0xb8;r0xaf")[0]&2?true:false;
  }
  void set_auto_gain(int value) {
    program("w0xcc02=1;w0xcc01=0xb8;W0xaf(2)=?",value?2:0);
  }
  int get_sensor_width() {
    return 752;
  }
  int get_sensor_height() {
    return 480;
  }
  int get_sensor_count() {
    return 5;
  }
  int get_color() {
    char cn[] = {serial().end()[-1],0};
    return atoi(cn)&1;
  }
  int get_stitching_enabled() {
    return stitching_enabled?1:0;
  }
  void set_stitching_enabled(int value) {
    stitching_enabled = !!value;
  }
  int get_trigger_mode() {
    return program("w2=2;w1=0x94;r8004")[0];
  }
  void set_trigger_mode(int value) {
    program("w2=2;w1=0x94;w8004=?",value);
  }
  int get_max_processing_queued_frames() {
    return cpu_pipeline.maxPending();
  }
  void set_max_processing_queued_frames(int value) {
    cpu_pipeline.setMaxPending(value);
  }

  OccamImage* run_cpu_pipeline(const OccamImage* image0) {
    OccamImage* image = (OccamImage*)image0;

    int si_x[] = {0,0,0,0,0};
    int si_y[] = {0*480,2*480,4*480,1*480,3*480};
    int si_width[] = {752,752,752,752,752};
    int si_height[] = {480,480,480,480,480};

    if (!stitching_enabled) {
      int offsetx[] = {0,0,0,0,0};
      int offsety[] = {0,0,0,0,0};
      image = simpleCPUBlend(image, 5,
			     si_x, si_y, si_width, si_height,
			     offsetx, offsety);
    } else {
      int offsetx[] = {0,0,0,0,0};
      int offsety[] = {0,0,0,0,0};
      image = simpleCPUBlend(image, 5,
			     si_x, si_y, si_width, si_height,
			     offsetx, offsety);
    }

    return image;
  }
public:
  OccamDevice_omni5u3mt9v022(const std::string& cid)
    : OmniDevice(cid),
      stitching_enabled(false),
      cpu_pipeline([this](const OccamImage* image){
	  return this->run_cpu_pipeline(image);
	}) {
    using namespace std::placeholders;
    registerParam(OCCAM_EXPOSURE,"exposure",1,480,
		  std::bind(&OccamDevice_omni5u3mt9v022::get_exposure,this),
		  std::bind(&OccamDevice_omni5u3mt9v022::set_exposure,this,_1));
    registerParam(OCCAM_AUTO_EXPOSURE,"auto_exposure",0,1,
		  std::bind(&OccamDevice_omni5u3mt9v022::get_auto_exposure,this),
		  std::bind(&OccamDevice_omni5u3mt9v022::set_auto_exposure,this,_1));
    registerParam(OCCAM_GAIN,"gain",1,64,
		  std::bind(&OccamDevice_omni5u3mt9v022::get_gain,this),
		  std::bind(&OccamDevice_omni5u3mt9v022::set_gain,this,_1));
    registerParam(OCCAM_AUTO_GAIN,"auto_gain",0,1,
		  std::bind(&OccamDevice_omni5u3mt9v022::get_auto_gain,this),
		  std::bind(&OccamDevice_omni5u3mt9v022::set_auto_gain,this,_1));
    registerParam(OCCAM_SENSOR_WIDTH,"sensor_width",0,0,
		  std::bind(&OccamDevice_omni5u3mt9v022::get_sensor_width,this));
    registerParam(OCCAM_SENSOR_HEIGHT,"sensor_height",0,0,
		  std::bind(&OccamDevice_omni5u3mt9v022::get_sensor_height,this));
    registerParam(OCCAM_SENSOR_COUNT,"sensor_count",0,0,
		  std::bind(&OccamDevice_omni5u3mt9v022::get_sensor_count,this));
    registerParam(OCCAM_COLOR,"color",0,0,
		  std::bind(&OccamDevice_omni5u3mt9v022::get_color,this));
    registerParam(OCCAM_STITCHING_ENABLED,"stitching_enabled",0,0,
		  std::bind(&OccamDevice_omni5u3mt9v022::get_stitching_enabled,this),
		  std::bind(&OccamDevice_omni5u3mt9v022::set_stitching_enabled,this,_1));
    registerParam(OCCAM_TRIGGER_MODE,"trigger_mode",0,0,
		  std::bind(&OccamDevice_omni5u3mt9v022::get_trigger_mode,this),
		  std::bind(&OccamDevice_omni5u3mt9v022::set_trigger_mode,this,_1)); 
    registerParam(OCCAM_MAX_PROCESSING_QUEUED_FRAMES,"max_processing_queued_frames",0,0,
		  std::bind(&OccamDevice_omni5u3mt9v022::get_max_processing_queued_frames,this),
		  std::bind(&OccamDevice_omni5u3mt9v022::set_max_processing_queued_frames,this,_1)); 

    program("w0xcc02=1;w0xcc01=0xb8;W0xd(0x30)=0x30");
  }

  virtual int readImage(OccamImage** image, int block) {
    for (;;) {
      int r = OmniDevice::readImage(image, block);
      if (r != OCCAM_API_SUCCESS && r != OCCAM_API_IMAGE_NOT_AVAILABLE)
	return r;
      if (r == OCCAM_API_SUCCESS)
	cpu_pipeline.push(*image);

      if (cpu_pipeline.pop(*image))
	return OCCAM_API_SUCCESS;
      else if (!block)
	return OCCAM_API_IMAGE_NOT_AVAILABLE;
    }    
  }
};

void init_omni5u3mt9v022() {
  registerDevice<OccamDevice_omni5u3mt9v022>("omni5u3mt9v022");
}
